@@ -1,13 +1,3 @@
-// Copyright 2012-2014 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
 use std::error::Error;
 use std::env;
 use std::fs;
@@ -30,13 +20,13 @@ fn main() {
     }
 }
 
-fn check_directory(dir: &Path) -> Result<(), Box<Error>> {
+fn check_directory(dir: &Path) -> Result<(), Box<dyn Error>> {
     for entry in fs::read_dir(dir)? {
         let entry = entry?;
         let path = entry.path();
 
         if path.is_dir() {
-            continue;
+            return check_directory(&path);
         }
 
         let mut file = File::open(&path)?;
@@ -44,10 +34,7 @@ fn check_directory(dir: &Path) -> Result<(), Box<Error>> {
         file.read_to_string(&mut contents)?;
 
         if contents.contains("#![feature") {
-            // attributes.md contains this and it is legitimate
-            if !contents.contains("#![feature(feature1, feature2, feature3)]") {
-                return Err(From::from(format!("Feature flag found in {:?}", path)));
-            }
+            return Err(From::from(format!("Feature flag found in {:?}", path)));
         }
     }
 
