@@ -1,18 +1,16 @@
 #![allow(rustc::default_hash_types)]
-
-#![recursion_limit="128"]
-
-extern crate proc_macro;
+#![recursion_limit = "128"]
 
 use synstructure::decl_derive;
 
 use proc_macro::TokenStream;
 
 mod hash_stable;
-mod type_foldable;
 mod lift;
 mod query;
+mod serialize;
 mod symbols;
+mod type_foldable;
 
 #[proc_macro]
 pub fn rustc_queries(input: TokenStream) -> TokenStream {
@@ -30,5 +28,11 @@ decl_derive!(
     hash_stable::hash_stable_generic_derive
 );
 
+decl_derive!([Decodable] => serialize::decodable_derive);
+decl_derive!([Encodable] => serialize::encodable_derive);
+decl_derive!([TyDecodable] => serialize::type_decodable_derive);
+decl_derive!([TyEncodable] => serialize::type_encodable_derive);
+decl_derive!([MetadataDecodable] => serialize::meta_decodable_derive);
+decl_derive!([MetadataEncodable] => serialize::meta_encodable_derive);
 decl_derive!([TypeFoldable, attributes(type_foldable)] => type_foldable::type_foldable_derive);
 decl_derive!([Lift, attributes(lift)] => lift::lift_derive);
