@@ -1,14 +1,14 @@
-use rustc::mir::*;
-use rustc::ty;
-use rustc_errors::{DiagnosticBuilder,Applicability};
-use syntax_pos::Span;
+use rustc_errors::{Applicability, DiagnosticBuilder};
+use rustc_middle::mir::*;
+use rustc_middle::ty;
+use rustc_span::source_map::DesugaringKind;
+use rustc_span::{sym, Span};
 
-use crate::borrow_check::MirBorrowckCtxt;
-use crate::borrow_check::prefixes::PrefixSet;
 use crate::borrow_check::diagnostics::UseSpans;
+use crate::borrow_check::prefixes::PrefixSet;
+use crate::borrow_check::MirBorrowckCtxt;
 use crate::dataflow::move_paths::{
-    IllegalMoveOrigin, IllegalMoveOriginKind,
-    LookupResult, MoveError, MovePathIndex,
+    IllegalMoveOrigin, IllegalMoveOriginKind, LookupResult, MoveError, MovePathIndex,
 };
 
 // Often when desugaring a pattern match we may have many individual moves in
@@ -62,7 +62,7 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
 
     fn group_move_errors(
         &self,
-        errors: Vec<(Place<'tcx>, MoveError<'tcx>)>
+        errors: Vec<(Place<'tcx>, MoveError<'tcx>)>,
     ) -> Vec<GroupedMoveError<'tcx>> {
         let mut grouped_errors = Vec::new();
         for (original_path, error) in errors {
@@ -81,16 +81,15 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
             MoveError::UnionMove { .. } => {
                 unimplemented!("don't know how to report union move errors yet.")
             }
-            MoveError::IllegalMove {
-                cannot_move_out_of: IllegalMoveOrigin { location, kind },
-            } => {
+            MoveError::IllegalMove { cannot_move_out_of: IllegalMoveOrigin { location, kind } } => {
                 // Note: that the only time we assign a place isn't a temporary
                 // to a user variable is when initializing it.
                 // If that ever stops being the case, then the ever initialized
                 // flow could be used.
-                if let Some(StatementKind::Assign(
-                    box(place, Rvalue::Use(Operand::Move(move_from)))
-                )) = self.body.basic_blocks()[location.block]
+                if let Some(StatementKind::Assign(box (
+                    place,
+                    Rvalue::Use(Operand::Move(move_from)),
+                ))) = self.body.basic_blocks()[location.block]
                     .statements
                     .get(location.statement_index)
                     .map(|stmt| &stmt.kind)
@@ -104,20 +103,21 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
                         //
                         // opt_match_place is None for let [mut] x = ... statements,
                         // whether or not the right-hand side is a place expression
-                        if let LocalInfo::User(ClearCrossCrate::Set(BindingForm::Var(
+                        if let Some(box LocalInfo::User(ClearCrossCrate::Set(BindingForm::Var(
                             VarBindingForm {
-                                opt_match_place: Some((ref opt_match_place, match_span)),
+                                opt_match_place: Some((opt_match_place, match_span)),
                                 binding_mode: _,
                                 opt_ty_info: _,
                                 pat_span: _,
                             },
-                        ))) = local_decl.local_info {
+                        )))) = local_decl.local_info
+                        {
                             let stmt_source_info = self.body.source_info(location);
                             self.append_binding_error(
                                 grouped_errors,
                                 kind,
                                 original_path,
-                                move_from,
+                                *move_from,
                                 local,
                                 opt_match_place,
                                 match_span,
@@ -143,19 +143,16 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
         grouped_errors: &mut Vec<GroupedMoveError<'tcx>>,
         kind: IllegalMoveOriginKind<'tcx>,
         original_path: Place<'tcx>,
-        move_from: &Place<'tcx>,
+        move_from: Place<'tcx>,
         bind_to: Local,
-        match_place: &Option<Place<'tcx>>,
+        match_place: Option<Place<'tcx>>,
         match_span: Span,
         statement_span: Span,
     ) {
-        debug!(
-            "append_binding_error(match_place={:?}, match_span={:?})",
-            match_place, match_span
-        );
+        debug!("append_binding_error(match_place={:?}, match_span={:?})", match_place, match_span);
 
         let from_simple_let = match_place.is_none();
-        let match_place = match_place.as_ref().unwrap_or(move_from);
+        let match_place = match_place.unwrap_or(move_from);
 
         match self.move_data.rev_lookup.find(match_place.as_ref()) {
             // Error with the match place
@@ -181,7 +178,7 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
                 };
                 grouped_errors.push(GroupedMoveError::MovesFromPlace {
                     span,
-                    move_from: match_place.clone(),
+                    move_from,
                     original_path,
                     kind,
                     binds_to,
@@ -223,49 +220,43 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
 
     fn report(&mut self, error: GroupedMoveError<'tcx>) {
         let (mut err, err_span) = {
-            let (span, use_spans, original_path, kind,):
-            (
+            let (span, use_spans, original_path, kind): (
                 Span,
                 Option<UseSpans>,
-                &Place<'tcx>,
+                Place<'tcx>,
                 &IllegalMoveOriginKind<'_>,
-            ) =
-                match error {
-                    GroupedMoveError::MovesFromPlace { span, ref original_path, ref kind, .. } |
-                    GroupedMoveError::MovesFromValue { span, ref original_path, ref kind, .. } => {
-                        (span, None, original_path, kind)
-                    }
-                    GroupedMoveError::OtherIllegalMove {
-                        use_spans,
-                        ref original_path,
-                        ref kind
-                    } => {
-                        (use_spans.args_or_use(), Some(use_spans), original_path, kind)
-                    },
-                };
-            debug!("report: original_path={:?} span={:?}, kind={:?} \
-                   original_path.is_upvar_field_projection={:?}", original_path, span, kind,
-                   self.is_upvar_field_projection(original_path.as_ref()));
+            ) = match error {
+                GroupedMoveError::MovesFromPlace { span, original_path, ref kind, .. }
+                | GroupedMoveError::MovesFromValue { span, original_path, ref kind, .. } => {
+                    (span, None, original_path, kind)
+                }
+                GroupedMoveError::OtherIllegalMove { use_spans, original_path, ref kind } => {
+                    (use_spans.args_or_use(), Some(use_spans), original_path, kind)
+                }
+            };
+            debug!(
+                "report: original_path={:?} span={:?}, kind={:?} \
+                   original_path.is_upvar_field_projection={:?}",
+                original_path,
+                span,
+                kind,
+                self.is_upvar_field_projection(original_path.as_ref())
+            );
             (
                 match kind {
-                    IllegalMoveOriginKind::Static => {
-                        unreachable!();
-                    }
-                    IllegalMoveOriginKind::BorrowedContent { target_place } => {
-                        self.report_cannot_move_from_borrowed_content(
+                    IllegalMoveOriginKind::BorrowedContent { target_place } => self
+                        .report_cannot_move_from_borrowed_content(
                             original_path,
-                            target_place,
+                            *target_place,
                             span,
                             use_spans,
-                        )
-                    }
+                        ),
                     IllegalMoveOriginKind::InteriorOfTypeWithDestructor { container_ty: ty } => {
                         self.cannot_move_out_of_interior_of_drop(span, ty)
                     }
-                    IllegalMoveOriginKind::InteriorOfSliceOrArray { ty, is_index } =>
-                        self.cannot_move_out_of_interior_noncopy(
-                            span, ty, Some(*is_index),
-                        ),
+                    IllegalMoveOriginKind::InteriorOfSliceOrArray { ty, is_index } => {
+                        self.cannot_move_out_of_interior_noncopy(span, ty, Some(*is_index))
+                    }
                 },
                 span,
             )
@@ -277,21 +268,18 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
 
     fn report_cannot_move_from_static(
         &mut self,
-        place: &Place<'tcx>,
-        span: Span
+        place: Place<'tcx>,
+        span: Span,
     ) -> DiagnosticBuilder<'a> {
         let description = if place.projection.len() == 1 {
-            format!("static item `{}`", self.describe_place(place.as_ref()).unwrap())
+            format!("static item {}", self.describe_any_place(place.as_ref()))
         } else {
-            let base_static = PlaceRef {
-                base: &place.base,
-                projection: &[ProjectionElem::Deref],
-            };
+            let base_static = PlaceRef { local: place.local, projection: &[ProjectionElem::Deref] };
 
             format!(
-                "`{:?}` as `{:?}` is a static item",
-                self.describe_place(place.as_ref()).unwrap(),
-                self.describe_place(base_static).unwrap(),
+                "{} as {} is a static item",
+                self.describe_any_place(place.as_ref()),
+                self.describe_any_place(base_static),
             )
         };
 
@@ -300,41 +288,37 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
 
     fn report_cannot_move_from_borrowed_content(
         &mut self,
-        move_place: &Place<'tcx>,
-        deref_target_place: &Place<'tcx>,
+        move_place: Place<'tcx>,
+        deref_target_place: Place<'tcx>,
         span: Span,
         use_spans: Option<UseSpans>,
     ) -> DiagnosticBuilder<'a> {
         // Inspect the type of the content behind the
         // borrow to provide feedback about why this
         // was a move rather than a copy.
-        let ty = deref_target_place.ty(*self.body, self.infcx.tcx).ty;
-        let upvar_field = self.prefixes(move_place.as_ref(), PrefixSet::All)
+        let ty = deref_target_place.ty(self.body, self.infcx.tcx).ty;
+        let upvar_field = self
+            .prefixes(move_place.as_ref(), PrefixSet::All)
             .find_map(|p| self.is_upvar_field_projection(p));
 
         let deref_base = match deref_target_place.projection.as_ref() {
             &[ref proj_base @ .., ProjectionElem::Deref] => {
-                PlaceRef {
-                    base: &deref_target_place.base,
-                    projection: &proj_base,
-                }
+                PlaceRef { local: deref_target_place.local, projection: &proj_base }
             }
             _ => bug!("deref_target_place is not a deref projection"),
         };
 
-        if let PlaceRef {
-            base: PlaceBase::Local(local),
-            projection: [],
-        } = deref_base {
-            let decl = &self.body.local_decls[*local];
+        if let PlaceRef { local, projection: [] } = deref_base {
+            let decl = &self.body.local_decls[local];
             if decl.is_ref_for_guard() {
                 let mut err = self.cannot_move_out_of(
                     span,
-                    &format!("`{}` in pattern guard", self.local_names[*local].unwrap()),
+                    &format!("`{}` in pattern guard", self.local_names[local].unwrap()),
                 );
                 err.note(
                     "variables bound in patterns cannot be moved from \
-                     until after the end of the pattern guard");
+                     until after the end of the pattern guard",
+                );
                 return err;
             } else if decl.is_ref_to_static() {
                 return self.report_cannot_move_from_static(move_place, span);
@@ -343,21 +327,17 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
 
         debug!("report: ty={:?}", ty);
         let mut err = match ty.kind {
-            ty::Array(..) | ty::Slice(..) =>
-                self.cannot_move_out_of_interior_noncopy(span, ty, None),
+            ty::Array(..) | ty::Slice(..) => {
+                self.cannot_move_out_of_interior_noncopy(span, ty, None)
+            }
             ty::Closure(def_id, closure_substs)
-                if def_id == self.mir_def_id && upvar_field.is_some()
-            => {
-                let closure_kind_ty = closure_substs
-                    .as_closure().kind_ty(def_id, self.infcx.tcx);
+                if def_id.as_local() == Some(self.mir_def_id) && upvar_field.is_some() =>
+            {
+                let closure_kind_ty = closure_substs.as_closure().kind_ty();
                 let closure_kind = closure_kind_ty.to_opt_closure_kind();
                 let capture_description = match closure_kind {
-                    Some(ty::ClosureKind::Fn) => {
-                        "captured variable in an `Fn` closure"
-                    }
-                    Some(ty::ClosureKind::FnMut) => {
-                        "captured variable in an `FnMut` closure"
-                    }
+                    Some(ty::ClosureKind::Fn) => "captured variable in an `Fn` closure",
+                    Some(ty::ClosureKind::FnMut) => "captured variable in an `FnMut` closure",
                     Some(ty::ClosureKind::FnOnce) => {
                         bug!("closure kind does not match first argument type")
                     }
@@ -369,21 +349,14 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
                 let upvar_name = upvar.name;
                 let upvar_span = self.infcx.tcx.hir().span(upvar_hir_id);
 
-                let place_name = self.describe_place(move_place.as_ref()).unwrap();
+                let place_name = self.describe_any_place(move_place.as_ref());
 
-                let place_description = if self
-                    .is_upvar_field_projection(move_place.as_ref())
-                    .is_some()
-                {
-                    format!("`{}`, a {}", place_name, capture_description)
-                } else {
-                    format!(
-                        "`{}`, as `{}` is a {}",
-                        place_name,
-                        upvar_name,
-                        capture_description,
-                    )
-                };
+                let place_description =
+                    if self.is_upvar_field_projection(move_place.as_ref()).is_some() {
+                        format!("{}, a {}", place_name, capture_description)
+                    } else {
+                        format!("{}, as `{}` is a {}", place_name, upvar_name, capture_description)
+                    };
 
                 debug!(
                     "report: closure_kind_ty={:?} closure_kind={:?} place_description={:?}",
@@ -398,43 +371,51 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
             }
             _ => {
                 let source = self.borrowed_content_source(deref_base);
-                match (
-                    self.describe_place(move_place.as_ref()),
-                    source.describe_for_named_place(),
-                ) {
-                    (Some(place_desc), Some(source_desc)) => {
-                        self.cannot_move_out_of(
-                            span,
-                            &format!("`{}` which is behind a {}", place_desc, source_desc),
-                        )
-                    }
-                    (_, _) => {
-                        self.cannot_move_out_of(
-                            span,
-                            &source.describe_for_unnamed_place(),
-                        )
-                    }
+                match (self.describe_place(move_place.as_ref()), source.describe_for_named_place())
+                {
+                    (Some(place_desc), Some(source_desc)) => self.cannot_move_out_of(
+                        span,
+                        &format!("`{}` which is behind a {}", place_desc, source_desc),
+                    ),
+                    (_, _) => self.cannot_move_out_of(
+                        span,
+                        &source.describe_for_unnamed_place(self.infcx.tcx),
+                    ),
                 }
-            },
+            }
         };
-        let move_ty = format!(
-            "{:?}",
-            move_place.ty(*self.body, self.infcx.tcx).ty,
-        );
         if let Ok(snippet) = self.infcx.tcx.sess.source_map().span_to_snippet(span) {
-            let is_option = move_ty.starts_with("std::option::Option");
-            let is_result = move_ty.starts_with("std::result::Result");
+            let def_id = match move_place.ty(self.body, self.infcx.tcx).ty.kind {
+                ty::Adt(self_def, _) => self_def.did,
+                ty::Foreign(def_id)
+                | ty::FnDef(def_id, _)
+                | ty::Closure(def_id, _)
+                | ty::Generator(def_id, ..)
+                | ty::Opaque(def_id, _) => def_id,
+                _ => return err,
+            };
+            let is_option = self.infcx.tcx.is_diagnostic_item(sym::option_type, def_id);
+            let is_result = self.infcx.tcx.is_diagnostic_item(sym::result_type, def_id);
             if (is_option || is_result) && use_spans.map_or(true, |v| !v.for_closure()) {
                 err.span_suggestion(
                     span,
-                    &format!("consider borrowing the `{}`'s content", if is_option {
-                        "Option"
-                    } else {
-                        "Result"
-                    }),
+                    &format!(
+                        "consider borrowing the `{}`'s content",
+                        if is_option { "Option" } else { "Result" }
+                    ),
                     format!("{}.as_ref()", snippet),
                     Applicability::MaybeIncorrect,
                 );
+            } else if matches!(span.desugaring_kind(), Some(DesugaringKind::ForLoop(_)))
+                && self.infcx.tcx.is_diagnostic_item(sym::vec_type, def_id)
+            {
+                // FIXME: suggest for anything that implements `IntoIterator`.
+                err.span_suggestion(
+                    span,
+                    "consider iterating over a slice of the `Vec<_>`'s content",
+                    format!("&{}", snippet),
+                    Applicability::MaybeIncorrect,
+                );
             }
         }
         err
@@ -447,11 +428,7 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
         span: Span,
     ) {
         match error {
-            GroupedMoveError::MovesFromPlace {
-                mut binds_to,
-                move_from,
-                ..
-            } => {
+            GroupedMoveError::MovesFromPlace { mut binds_to, move_from, .. } => {
                 if let Ok(snippet) = self.infcx.tcx.sess.source_map().span_to_snippet(span) {
                     err.span_suggestion(
                         span,
@@ -462,18 +439,13 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
                 }
 
                 if binds_to.is_empty() {
-                    let place_ty = move_from.ty(*self.body, self.infcx.tcx).ty;
+                    let place_ty = move_from.ty(self.body, self.infcx.tcx).ty;
                     let place_desc = match self.describe_place(move_from.as_ref()) {
                         Some(desc) => format!("`{}`", desc),
-                        None => format!("value"),
+                        None => "value".to_string(),
                     };
 
-                    self.note_type_does_not_implement_copy(
-                        err,
-                        &place_desc,
-                        place_ty,
-                        Some(span)
-                    );
+                    self.note_type_does_not_implement_copy(err, &place_desc, place_ty, Some(span));
                 } else {
                     binds_to.sort();
                     binds_to.dedup();
@@ -490,61 +462,40 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
             // No binding. Nothing to suggest.
             GroupedMoveError::OtherIllegalMove { ref original_path, use_spans, .. } => {
                 let span = use_spans.var_or_use();
-                let place_ty = original_path.ty(*self.body, self.infcx.tcx).ty;
+                let place_ty = original_path.ty(self.body, self.infcx.tcx).ty;
                 let place_desc = match self.describe_place(original_path.as_ref()) {
                     Some(desc) => format!("`{}`", desc),
-                    None => format!("value"),
+                    None => "value".to_string(),
                 };
-                self.note_type_does_not_implement_copy(
-                    err,
-                    &place_desc,
-                    place_ty,
-                    Some(span),
-                );
+                self.note_type_does_not_implement_copy(err, &place_desc, place_ty, Some(span));
 
                 use_spans.args_span_label(err, format!("move out of {} occurs here", place_desc));
-                use_spans.var_span_label(
-                    err,
-                    format!("move occurs due to use{}", use_spans.describe()),
-                );
-            },
+                use_spans
+                    .var_span_label(err, format!("move occurs due to use{}", use_spans.describe()));
+            }
         }
     }
 
-    fn add_move_error_suggestions(
-        &self,
-        err: &mut DiagnosticBuilder<'a>,
-        binds_to: &[Local],
-    ) {
+    fn add_move_error_suggestions(&self, err: &mut DiagnosticBuilder<'a>, binds_to: &[Local]) {
         let mut suggestions: Vec<(Span, &str, String)> = Vec::new();
         for local in binds_to {
             let bind_to = &self.body.local_decls[*local];
-            if let LocalInfo::User(
-                ClearCrossCrate::Set(BindingForm::Var(VarBindingForm {
-                    pat_span,
-                    ..
-                }))
-            ) = bind_to.local_info {
+            if let Some(box LocalInfo::User(ClearCrossCrate::Set(BindingForm::Var(
+                VarBindingForm { pat_span, .. },
+            )))) = bind_to.local_info
+            {
                 if let Ok(pat_snippet) = self.infcx.tcx.sess.source_map().span_to_snippet(pat_span)
                 {
                     if pat_snippet.starts_with('&') {
                         let pat_snippet = pat_snippet[1..].trim_start();
-                        let suggestion;
-                        let to_remove;
-                        if pat_snippet.starts_with("mut")
+                        let (suggestion, to_remove) = if pat_snippet.starts_with("mut")
                             && pat_snippet["mut".len()..].starts_with(rustc_lexer::is_whitespace)
                         {
-                            suggestion = pat_snippet["mut".len()..].trim_start();
-                            to_remove = "&mut";
+                            (pat_snippet["mut".len()..].trim_start(), "&mut")
                         } else {
-                            suggestion = pat_snippet;
-                            to_remove = "&";
-                        }
-                        suggestions.push((
-                            pat_span,
-                            to_remove,
-                            suggestion.to_owned(),
-                        ));
+                            (pat_snippet, "&")
+                        };
+                        suggestions.push((pat_span, to_remove, suggestion.to_owned()));
                     }
                 }
             }
@@ -561,12 +512,8 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
         }
     }
 
-    fn add_move_error_details(
-        &self,
-        err: &mut DiagnosticBuilder<'a>,
-        binds_to: &[Local],
-    ) {
-        for (j, local) in binds_to.into_iter().enumerate() {
+    fn add_move_error_details(&self, err: &mut DiagnosticBuilder<'a>, binds_to: &[Local]) {
+        for (j, local) in binds_to.iter().enumerate() {
             let bind_to = &self.body.local_decls[*local];
             let binding_span = bind_to.source_info.span;
 
@@ -581,13 +528,14 @@ impl<'a, 'tcx> MirBorrowckCtxt<'a, 'tcx> {
                     err,
                     &format!("`{}`", self.local_names[*local].unwrap()),
                     bind_to.ty,
-                    Some(binding_span)
+                    Some(binding_span),
                 );
             }
         }
 
         if binds_to.len() > 1 {
-            err.note("move occurs because these variables have types that \
+            err.note(
+                "move occurs because these variables have types that \
                       don't implement the `Copy` trait",
             );
         }
