@@ -1,22 +1,19 @@
 use crate::abi::call::{ArgAbi, FnAbi, Reg};
 use crate::abi::Abi;
 
-// Win64 ABI: http://msdn.microsoft.com/en-us/library/zthk2dkh.aspx
+// Win64 ABI: https://docs.microsoft.com/en-us/cpp/build/parameter-passing
 
 pub fn compute_abi_info<Ty>(fn_abi: &mut FnAbi<'_, Ty>) {
     let fixup = |a: &mut ArgAbi<'_, Ty>| {
         match a.layout.abi {
             Abi::Uninhabited => {}
-            Abi::ScalarPair(..) |
-            Abi::Aggregate { .. } => {
-                match a.layout.size.bits() {
-                    8 => a.cast_to(Reg::i8()),
-                    16 => a.cast_to(Reg::i16()),
-                    32 => a.cast_to(Reg::i32()),
-                    64 => a.cast_to(Reg::i64()),
-                    _ => a.make_indirect()
-                }
-            }
+            Abi::ScalarPair(..) | Abi::Aggregate { .. } => match a.layout.size.bits() {
+                8 => a.cast_to(Reg::i8()),
+                16 => a.cast_to(Reg::i16()),
+                32 => a.cast_to(Reg::i32()),
+                64 => a.cast_to(Reg::i64()),
+                _ => a.make_indirect(),
+            },
             Abi::Vector { .. } => {
                 // FIXME(eddyb) there should be a size cap here
                 // (probably what clang calls "illegal vectors").
@@ -35,7 +32,9 @@ pub fn compute_abi_info<Ty>(fn_abi: &mut FnAbi<'_, Ty>) {
         fixup(&mut fn_abi.ret);
     }
     for arg in &mut fn_abi.args {
-        if arg.is_ignore() { continue; }
+        if arg.is_ignore() {
+            continue;
+        }
         fixup(arg);
     }
 }
