@@ -8,7 +8,7 @@ pub fn target() -> TargetResult {
         target_endian: "little".to_string(),
         target_pointer_width: "32".to_string(),
         target_c_int_width: "32".to_string(),
-        data_layout: "e-m:e-p:32:32-i64:64-v128:64:128-a:0:32-n32-S64".to_string(),
+        data_layout: "e-m:e-p:32:32-Fi8-i64:64-v128:64:128-a:0:32-n32-S64".to_string(),
         arch: "arm".to_string(),
         target_os: "netbsd".to_string(),
         target_env: "eabihf".to_string(),
@@ -16,9 +16,10 @@ pub fn target() -> TargetResult {
         linker_flavor: LinkerFlavor::Gcc,
 
         options: TargetOptions {
-            features: "+v6,+vfp2".to_string(),
-            abi_blacklist: super::arm_base::abi_blacklist(),
-            .. base
-        }
+            features: "+v6,+vfp2,-d32".to_string(),
+            unsupported_abis: super::arm_base::unsupported_abis(),
+            target_mcount: "__mcount".to_string(),
+            ..base
+        },
     })
 }
