@@ -1,13 +1,15 @@
-use crate::spec::{LinkArgs, LinkerFlavor, TargetOptions, RelroLevel};
+use crate::spec::{LinkArgs, LinkerFlavor, RelroLevel, TargetOptions, TlsModel};
 
 pub fn opts() -> TargetOptions {
     let mut args = LinkArgs::new();
-    args.insert(LinkerFlavor::Gcc, vec![
-        "-Wl,-Bstatic".to_string(),
-        "-Wl,--no-dynamic-linker".to_string(),
-        "-Wl,--eh-frame-hdr".to_string(),
-        "-Wl,--gc-sections".to_string(),
-    ]);
+    args.insert(
+        LinkerFlavor::Gcc,
+        vec![
+            "-Wl,-Bstatic".to_string(),
+            "-Wl,--no-dynamic-linker".to_string(),
+            "-Wl,--gc-sections".to_string(),
+        ],
+    );
 
     TargetOptions {
         executables: true,
@@ -26,8 +28,8 @@ pub fn opts() -> TargetOptions {
         // (Global Offset Table) to obtain the effective address of a
         // thread-local variable. Using a GOT is useful only when doing
         // dynamic linking.
-        tls_model: "local-exec".to_string(),
+        tls_model: TlsModel::LocalExec,
         relro_level: RelroLevel::Full,
-        .. Default::default()
+        ..Default::default()
     }
 }
