@@ -17,9 +17,6 @@ pub fn target() -> TargetResult {
         target_env: "musl".to_string(),
         target_vendor: "unknown".to_string(),
         linker_flavor: LinkerFlavor::Gcc,
-        options: TargetOptions {
-            target_mcount: "_mcount".to_string(),
-            .. base
-        },
+        options: TargetOptions { target_mcount: "_mcount".to_string(), ..base },
     })
 }
