@@ -1,5 +1,5 @@
-use crate::spec::{LinkerFlavor, LldFlavor, PanicStrategy,
-           Target, TargetOptions, TargetResult};
+use crate::spec::{LinkerFlavor, LldFlavor, PanicStrategy, RelocModel};
+use crate::spec::{Target, TargetOptions, TargetResult};
 
 pub fn target() -> TargetResult {
     Ok(Target {
@@ -22,11 +22,11 @@ pub fn target() -> TargetResult {
             features: String::new(),
             executables: true,
             panic_strategy: PanicStrategy::Abort,
-            relocation_model: "static".to_string(),
+            relocation_model: RelocModel::Static,
             emit_debug_gdb_scripts: false,
-            abi_blacklist: super::riscv_base::abi_blacklist(),
-            eliminate_frame_pointer: false,
-            .. Default::default()
+            unsupported_abis: super::riscv_base::unsupported_abis(),
+            eh_frame_header: false,
+            ..Default::default()
         },
     })
 }
