@@ -1,5 +1,5 @@
-use crate::spec::{LinkerFlavor, LldFlavor, PanicStrategy,
-           Target, TargetOptions, TargetResult};
+use crate::spec::{CodeModel, Target, TargetOptions, TargetResult};
+use crate::spec::{LinkerFlavor, LldFlavor, PanicStrategy, RelocModel};
 
 pub fn target() -> TargetResult {
     Ok(Target {
@@ -22,10 +22,12 @@ pub fn target() -> TargetResult {
             features: "+m,+a,+c".to_string(),
             executables: true,
             panic_strategy: PanicStrategy::Abort,
-            relocation_model: "static".to_string(),
+            relocation_model: RelocModel::Static,
+            code_model: Some(CodeModel::Medium),
             emit_debug_gdb_scripts: false,
-            abi_blacklist: super::riscv_base::abi_blacklist(),
-            .. Default::default()
+            unsupported_abis: super::riscv_base::unsupported_abis(),
+            eh_frame_header: false,
+            ..Default::default()
         },
     })
 }
