@@ -4,32 +4,29 @@
 #![feature(crate_visibility_modifier)]
 #![feature(in_band_lifetimes)]
 #![feature(nll)]
-
-#![recursion_limit="256"]
+#![feature(or_patterns)]
+#![recursion_limit = "256"]
 
 #[macro_use]
-extern crate log;
+extern crate tracing;
 #[macro_use]
-extern crate rustc;
+extern crate rustc_middle;
 
-mod chalk_context;
+mod chalk;
 mod dropck_outlives;
 mod evaluate_obligation;
 mod implied_outlives_bounds;
-mod normalize_projection_ty;
 mod normalize_erasing_regions;
-pub mod lowering;
-mod generic_types;
+mod normalize_projection_ty;
 mod type_op;
 
-use rustc::ty::query::Providers;
+use rustc_middle::ty::query::Providers;
 
-pub fn provide(p: &mut Providers<'_>) {
+pub fn provide(p: &mut Providers) {
     dropck_outlives::provide(p);
     evaluate_obligation::provide(p);
     implied_outlives_bounds::provide(p);
-    lowering::provide(p);
-    chalk_context::provide(p);
+    chalk::provide(p);
     normalize_projection_ty::provide(p);
     normalize_erasing_regions::provide(p);
     type_op::provide(p);
