@@ -16,6 +16,19 @@
 using namespace llvm;
 using namespace llvm::orc;
 
+// Writing 64-bit XCOFF isn't supported yet, so we need to disable some tests on
+// AIX till it is.
+#ifdef _AIX
+#define SKIPTEST_IF_UNSUPPORTED()                                              \
+  do {                                                                         \
+    return;                                                                    \
+  } while (false)
+#else
+#define SKIPTEST_IF_UNSUPPORTED()                                              \
+  do {                                                                         \
+  } while (false)
+#endif
+
 namespace {
 
 class MockObjectLayer {
@@ -105,10 +118,11 @@ MockObjectLayer::ObjectPtr createTestObject() {
   B.CreateRet(ConstantInt::getSigned(Type::getInt32Ty(Ctx), 42));
 
   SimpleCompiler IRCompiler(*TM);
-  return IRCompiler(*MB.getModule());
+  return cantFail(IRCompiler(*MB.getModule()));
 }
 
 TEST(RemoteObjectLayer, AddObject) {
+  SKIPTEST_IF_UNSUPPORTED();
   llvm::orc::rpc::registerStringError<rpc::RawByteChannel>();
   auto TestObject = createTestObject();
   if (!TestObject)
@@ -166,6 +180,7 @@ TEST(RemoteObjectLayer, AddObject) {
 }
 
 TEST(RemoteObjectLayer, AddObjectFailure) {
+  SKIPTEST_IF_UNSUPPORTED();
   llvm::orc::rpc::registerStringError<rpc::RawByteChannel>();
   auto TestObject = createTestObject();
   if (!TestObject)
@@ -221,6 +236,7 @@ TEST(RemoteObjectLayer, AddObjectFailure) {
 
 
 TEST(RemoteObjectLayer, RemoveObject) {
+  SKIPTEST_IF_UNSUPPORTED();
   llvm::orc::rpc::registerStringError<rpc::RawByteChannel>();
   auto TestObject = createTestObject();
   if (!TestObject)
@@ -268,6 +284,7 @@ TEST(RemoteObjectLayer, RemoveObject) {
 }
 
 TEST(RemoteObjectLayer, RemoveObjectFailure) {
+  SKIPTEST_IF_UNSUPPORTED();
   llvm::orc::rpc::registerStringError<rpc::RawByteChannel>();
   auto TestObject = createTestObject();
   if (!TestObject)
@@ -324,6 +341,7 @@ TEST(RemoteObjectLayer, RemoveObjectFailure) {
 }
 
 TEST(RemoteObjectLayer, FindSymbol) {
+  SKIPTEST_IF_UNSUPPORTED();
   llvm::orc::rpc::registerStringError<rpc::RawByteChannel>();
   auto TestObject = createTestObject();
   if (!TestObject)
@@ -354,7 +372,7 @@ TEST(RemoteObjectLayer, FindSymbol) {
           if (Name == "foobar")
             return JITSymbol(0x12348765, JITSymbolFlags::Exported);
           if (Name == "badsymbol")
-            return make_error<JITSymbolNotFound>(Name);
+            return make_error<JITSymbolNotFound>(std::string(Name));
           return nullptr;
         };
       return 42;
@@ -406,6 +424,7 @@ TEST(RemoteObjectLayer, FindSymbol) {
 }
 
 TEST(RemoteObjectLayer, FindSymbolIn) {
+  SKIPTEST_IF_UNSUPPORTED();
   llvm::orc::rpc::registerStringError<rpc::RawByteChannel>();
   auto TestObject = createTestObject();
   if (!TestObject)
@@ -435,7 +454,7 @@ TEST(RemoteObjectLayer, FindSymbolIn) {
         [](StringRef Name, bool ExportedSymbolsOnly) -> JITSymbol {
           if (Name == "foobar")
             return JITSymbol(0x12348765, JITSymbolFlags::Exported);
-          return make_error<JITSymbolNotFound>(Name);
+          return make_error<JITSymbolNotFound>(std::string(Name));
         };
       // Dummy symbol table entry - this should not be visible to
       // findSymbolIn.
@@ -443,7 +462,7 @@ TEST(RemoteObjectLayer, FindSymbolIn) {
         [](StringRef Name, bool ExportedSymbolsOnly) -> JITSymbol {
           if (Name == "barbaz")
             return JITSymbol(0xdeadbeef, JITSymbolFlags::Exported);
-          return make_error<JITSymbolNotFound>(Name);
+          return make_error<JITSymbolNotFound>(std::string(Name));
         };
 
       return 42;
@@ -484,6 +503,7 @@ TEST(RemoteObjectLayer, FindSymbolIn) {
 }
 
 TEST(RemoteObjectLayer, EmitAndFinalize) {
+  SKIPTEST_IF_UNSUPPORTED();
   llvm::orc::rpc::registerStringError<rpc::RawByteChannel>();
   auto TestObject = createTestObject();
   if (!TestObject)
@@ -532,6 +552,7 @@ TEST(RemoteObjectLayer, EmitAndFinalize) {
 }
 
 TEST(RemoteObjectLayer, EmitAndFinalizeFailure) {
+  SKIPTEST_IF_UNSUPPORTED();
   llvm::orc::rpc::registerStringError<rpc::RawByteChannel>();
   auto TestObject = createTestObject();
   if (!TestObject)
