@@ -9,26 +9,21 @@ fn main() {
     // object code in the rustc executable).
     #[cfg(feature = "jemalloc-sys")]
     {
-        use std::os::raw::{c_void, c_int};
+        use std::os::raw::{c_int, c_void};
 
         #[used]
-        static _F1: unsafe extern fn(usize, usize) -> *mut c_void =
-            jemalloc_sys::calloc;
+        static _F1: unsafe extern "C" fn(usize, usize) -> *mut c_void = jemalloc_sys::calloc;
         #[used]
-        static _F2: unsafe extern fn(*mut *mut c_void, usize, usize) -> c_int =
+        static _F2: unsafe extern "C" fn(*mut *mut c_void, usize, usize) -> c_int =
             jemalloc_sys::posix_memalign;
         #[used]
-        static _F3: unsafe extern fn(usize, usize) -> *mut c_void =
-            jemalloc_sys::aligned_alloc;
+        static _F3: unsafe extern "C" fn(usize, usize) -> *mut c_void = jemalloc_sys::aligned_alloc;
         #[used]
-        static _F4: unsafe extern fn(usize) -> *mut c_void =
-            jemalloc_sys::malloc;
+        static _F4: unsafe extern "C" fn(usize) -> *mut c_void = jemalloc_sys::malloc;
         #[used]
-        static _F5: unsafe extern fn(*mut c_void, usize) -> *mut c_void =
-            jemalloc_sys::realloc;
+        static _F5: unsafe extern "C" fn(*mut c_void, usize) -> *mut c_void = jemalloc_sys::realloc;
         #[used]
-        static _F6: unsafe extern fn(*mut c_void) =
-            jemalloc_sys::free;
+        static _F6: unsafe extern "C" fn(*mut c_void) = jemalloc_sys::free;
     }
 
     rustc_driver::set_sigpipe_handler();
