@@ -1,5 +1,5 @@
-// compile-pass
-// compile-flags:--test
+// check-pass
+// compile-flags:--test --test-args --test-threads=1
 // normalize-stdout-test: "src/test/rustdoc-ui" -> "$$DIR"
 
 // Crates like core have doctests gated on `cfg(not(test))` so we need to make
@@ -20,3 +20,11 @@ pub struct Foo;
 /// ```
 #[cfg(not(test))]
 pub struct Foo;
+
+/// this doctest will be tested, but will not appear in documentation:
+///
+/// ```
+/// assert!(true)
+/// ```
+#[cfg(doctest)]
+pub struct Bar;
