@@ -1,23 +1,25 @@
 // run-pass
-// Testing that a libsyntax can parse modules with canonicalized base path
+// Testing that a librustc_ast can parse modules with canonicalized base path
 // ignore-cross-compile
+// ignore-remote
 
 #![feature(rustc_private)]
 
-extern crate syntax;
-extern crate syntax_expand;
+extern crate rustc_ast;
 extern crate rustc_parse;
+extern crate rustc_session;
+extern crate rustc_span;
 
 use rustc_parse::new_parser_from_file;
+use rustc_session::parse::ParseSess;
+use rustc_span::source_map::FilePathMapping;
 use std::path::Path;
-use syntax::sess::ParseSess;
-use syntax::source_map::FilePathMapping;
 
 #[path = "mod_dir_simple/test.rs"]
 mod gravy;
 
 pub fn main() {
-    syntax::with_default_globals(|| parse());
+    rustc_span::with_default_session_globals(|| parse());
 
     assert_eq!(gravy::foo(), 10);
 }
@@ -27,6 +29,6 @@ fn parse() {
 
     let path = Path::new(file!());
     let path = path.canonicalize().unwrap();
-    let mut parser = new_parser_from_file(&parse_session, &path);
+    let mut parser = new_parser_from_file(&parse_session, &path, None);
     let _ = parser.parse_crate_mod();
 }
