@@ -1,11 +1,9 @@
-// compile-pass
+// build-pass (FIXME(62277): could be check-pass?)
 // edition:2018
 //
 // Tests that we properly handle StorageDead/StorageLives for temporaries
 // created in async loop bodies.
 
-#![feature(async_await)]
-
 async fn bar() -> Option<()> {
     Some(())
 }
