@@ -1,10 +1,9 @@
-#![feature(optin_builtin_traits)]
-#![feature(overlapping_marker_traits)]
+#![feature(negative_impls)]
 
 use std::marker::Copy;
 
 enum TestE {
-  A
+    A,
 }
 
 struct MyType;
@@ -24,7 +23,7 @@ unsafe impl Send for [MyType] {}
 //~^ ERROR E0117
 
 unsafe impl Send for &'static [NotSync] {}
-//~^ ERROR E0117
+//~^ ERROR conflicting implementations of trait
+//~| ERROR only traits defined in the current crate
 
-fn main() {
-}
+fn main() {}
