@@ -10,7 +10,7 @@ fn double(x: usize) -> usize {
 const X: fn(usize) -> usize = double;
 
 const fn bar(x: fn(usize) -> usize, y: usize) -> usize {
-    x(y) //~ WARN skipping const checks
+    x(y)
 }
 
 const Y: usize = bar(X, 2); // FIXME: should fail to typeck someday
