@@ -1,6 +1,4 @@
-// compile-pass
-
-#![feature(const_let)]
+// build-pass (FIXME(62277): could be check-pass?)
 
 struct S(i32);
 
