@@ -1,12 +1,12 @@
 // Verify the compiler fails with an error on infinite function
 // recursions.
 
-fn generic<T>() {
+// build-fail
+
+fn generic<T>() { //~ WARN function cannot return without recursing
     generic::<Option<T>>();
 }
-//~^^^ ERROR reached the recursion limit while instantiating `generic::<std::option::Option<
-//~| WARN function cannot return without recursing
-
+//~^^ ERROR reached the recursion limit while instantiating `generic::<std::option::Option<
 
 
 fn main () {
