@@ -5,7 +5,8 @@ trait Future {
 
 use std::error::Error;
 
-fn foo() -> impl Future<Item=(), Error=Box<Error>> {
+fn foo() -> impl Future<Item=(), Error=Box<dyn Error>> {
+//~^ ERROR not satisfied
     Ok(())
 }
 
