@@ -4,7 +4,7 @@
 fn main() {
     match &&Some(5i32) {
         Some(n) => {
-            *n += 1; //~ ERROR cannot assign to immutable
+            *n += 1; //~ ERROR cannot assign to `*n` which is behind a `&` reference
             let _ = n;
         }
         None => {},
@@ -12,7 +12,7 @@ fn main() {
 
     match &mut &Some(5i32) {
         Some(n) => {
-            *n += 1; //~ ERROR cannot assign to immutable
+            *n += 1; //~ ERROR cannot assign to `*n` which is behind a `&` reference
             let _ = n;
         }
         None => {},
@@ -20,7 +20,7 @@ fn main() {
 
     match &&mut Some(5i32) {
         Some(n) => {
-            *n += 1; //~ ERROR cannot assign to immutable
+            *n += 1; //~ ERROR cannot assign to `*n` which is behind a `&` reference
             let _ = n;
         }
         None => {},
