@@ -20,6 +20,6 @@ impl Foo for Bar {
 
 fn main() {
     let x = box Bar { x: 10 };
-    let y: Box<Foo> = x as Box<Foo>;
+    let y: Box<dyn Foo> = x as Box<dyn Foo>;
     let _z = y.clone(); //~ ERROR no method named `clone` found
 }
