@@ -31,56 +31,55 @@
 //! platform-specific cfgs are allowed. Not sure yet how to deal with
 //! this in the long term.
 
-use std::path::Path;
 use std::iter::Iterator;
+use std::path::Path;
 
 // Paths that may contain platform-specific code.
 const EXCEPTION_PATHS: &[&str] = &[
     // std crates
-    "src/libpanic_abort",
-    "src/libpanic_unwind",
-    "src/libunwind",
+    "library/panic_abort",
+    "library/panic_unwind",
+    "library/unwind",
     // black_box implementation is LLVM-version specific and it uses
-    // target_os to tell targets with different LLVM-versions appart
+    // target_os to tell targets with different LLVM-versions apart
     // (e.g. `wasm32-unknown-emscripten` vs `wasm32-unknown-unknown`):
-    "src/libcore/hint.rs",
-    "src/libstd/sys/", // Platform-specific code for std lives here.
-                       // This has the trailing slash so that sys_common is not excepted.
-    "src/libstd/os", // Platform-specific public interfaces
-    "src/rtstartup", // Not sure what to do about this. magic stuff for mingw
-
+    "library/core/src/hint.rs",
+    "library/std/src/sys/", // Platform-specific code for std lives here.
+    // This has the trailing slash so that sys_common is not excepted.
+    "library/std/src/os", // Platform-specific public interfaces
+    "library/rtstartup",  // Not sure what to do about this. magic stuff for mingw
     // temporary exceptions
-    "src/libstd/lib.rs",
-    "src/libstd/path.rs",
-    "src/libstd/f32.rs",
-    "src/libstd/f64.rs",
+    "library/std/src/lib.rs",
+    "library/std/src/path.rs",
+    "library/std/src/f32.rs",
+    "library/std/src/f64.rs",
     // Integration test for platform-specific run-time feature detection:
-    "src/libstd/tests/run-time-detect.rs" ,
-    "src/libstd/net/test.rs",
-    "src/libstd/sys_common/mod.rs",
-    "src/libstd/sys_common/net.rs",
-    "src/libstd/sys_common/backtrace.rs",
-    "src/libterm", // Not sure how to make this crate portable, but test crate needs it.
-    "src/libtest", // Probably should defer to unstable `std::sys` APIs.
-    "src/libstd/sync/mpsc", // some tests are only run on non-emscripten
-
+    "library/std/tests/run-time-detect.rs",
+    "library/std/src/net/test.rs",
+    "library/std/src/sys_common/mod.rs",
+    "library/std/src/sys_common/net.rs",
+    "library/std/src/sys_common/backtrace.rs",
+    // panic_unwind shims
+    "library/std/src/panicking.rs",
+    "library/term", // Not sure how to make this crate portable, but test crate needs it.
+    "library/test", // Probably should defer to unstable `std::sys` APIs.
+    "library/std/src/sync/mpsc", // some tests are only run on non-emscripten
     // std testing crates, okay for now at least
-    "src/libcore/tests",
-    "src/liballoc/tests/lib.rs",
-
+    "library/core/tests",
+    "library/alloc/tests/lib.rs",
+    "library/alloc/benches/lib.rs",
     // The `VaList` implementation must have platform specific code.
     // The Windows implementation of a `va_list` is always a character
     // pointer regardless of the target architecture. As a result,
     // we must use `#[cfg(windows)]` to conditionally compile the
     // correct `VaList` structure for windows.
-    "src/libcore/ffi.rs",
-
+    "library/core/src/ffi.rs",
     // non-std crates
     "src/test",
     "src/tools",
     "src/librustc",
     "src/librustdoc",
-    "src/libsyntax",
+    "src/librustc_ast",
     "src/bootstrap",
 ];
 
@@ -91,10 +90,14 @@ pub fn check(path: &Path, bad: &mut bool) {
     super::walk(path, &mut super::filter_dirs, &mut |entry, contents| {
         let file = entry.path();
         let filestr = file.to_string_lossy().replace("\\", "/");
-        if !filestr.ends_with(".rs") { return }
+        if !filestr.ends_with(".rs") {
+            return;
+        }
 
         let is_exception_path = EXCEPTION_PATHS.iter().any(|s| filestr.contains(&**s));
-        if is_exception_path { return }
+        if is_exception_path {
+            return;
+        }
 
         check_cfgs(contents, &file, bad, &mut saw_target_arch, &mut saw_cfg_bang);
     });
@@ -103,8 +106,13 @@ pub fn check(path: &Path, bad: &mut bool) {
     assert!(saw_cfg_bang);
 }
 
-fn check_cfgs(contents: &str, file: &Path,
-              bad: &mut bool, saw_target_arch: &mut bool, saw_cfg_bang: &mut bool) {
+fn check_cfgs(
+    contents: &str,
+    file: &Path,
+    bad: &mut bool,
+    saw_target_arch: &mut bool,
+    saw_cfg_bang: &mut bool,
+) {
     // For now it's ok to have platform-specific code after 'mod tests'.
     let mod_tests_idx = find_test_mod(contents);
     let contents = &contents[..mod_tests_idx];
@@ -119,24 +127,29 @@ fn check_cfgs(contents: &str, file: &Path,
         let line_numbers = line_numbers.as_ref().expect("");
         let line = match line_numbers.binary_search(&idx) {
             Ok(_) => unreachable!(),
-            Err(i) => i + 1
+            Err(i) => i + 1,
         };
         tidy_error!(bad, "{}:{}: platform-specific cfg: {}", file.display(), line, cfg);
     };
 
     for (idx, cfg) in cfgs {
         // Sanity check that the parsing here works.
-        if !*saw_target_arch && cfg.contains("target_arch") { *saw_target_arch = true }
-        if !*saw_cfg_bang && cfg.contains("cfg!") { *saw_cfg_bang = true }
+        if !*saw_target_arch && cfg.contains("target_arch") {
+            *saw_target_arch = true
+        }
+        if !*saw_cfg_bang && cfg.contains("cfg!") {
+            *saw_cfg_bang = true
+        }
 
-        let contains_platform_specific_cfg =
-            cfg.contains("target_os")
+        let contains_platform_specific_cfg = cfg.contains("target_os")
             || cfg.contains("target_env")
             || cfg.contains("target_vendor")
             || cfg.contains("unix")
             || cfg.contains("windows");
 
-        if !contains_platform_specific_cfg { continue }
+        if !contains_platform_specific_cfg {
+            continue;
+        }
 
         let preceeded_by_doc_comment = {
             let pre_contents = &contents[..idx];
@@ -149,7 +162,9 @@ fn check_cfgs(contents: &str, file: &Path,
             }
         };
 
-        if preceeded_by_doc_comment { continue }
+        if preceeded_by_doc_comment {
+            continue;
+        }
 
         err(idx, cfg);
     }
@@ -161,9 +176,10 @@ fn find_test_mod(contents: &str) -> usize {
         let prev_newline_idx = contents[..mod_tests_idx].rfind('\n').unwrap_or(mod_tests_idx);
         let prev_newline_idx = contents[..prev_newline_idx].rfind('\n');
         if let Some(nl) = prev_newline_idx {
-            let prev_line = &contents[nl + 1 .. mod_tests_idx];
+            let prev_line = &contents[nl + 1..mod_tests_idx];
             if prev_line.contains("cfg(all(test, not(target_os")
-                || prev_line.contains("cfg(all(test, not(any(target_os") {
+                || prev_line.contains("cfg(all(test, not(any(target_os")
+            {
                 nl
             } else {
                 mod_tests_idx
@@ -183,7 +199,9 @@ fn parse_cfgs<'a>(contents: &'a str) -> Vec<(usize, &'a str)> {
     // that appear to be tokens followed by a parenthesis.
     let cfgs = candidate_cfg_idxs.filter(|i| {
         let pre_idx = i.saturating_sub(*i);
-        let succeeds_non_ident = !contents.as_bytes().get(pre_idx)
+        let succeeds_non_ident = !contents
+            .as_bytes()
+            .get(pre_idx)
             .cloned()
             .map(char::from)
             .map(char::is_alphanumeric)
@@ -191,10 +209,12 @@ fn parse_cfgs<'a>(contents: &'a str) -> Vec<(usize, &'a str)> {
         let contents_after = &contents[*i..];
         let first_paren = contents_after.find('(');
         let paren_idx = first_paren.map(|ip| i + ip);
-        let preceeds_whitespace_and_paren = paren_idx.map(|ip| {
-            let maybe_space = &contents[*i + "cfg".len() .. ip];
-            maybe_space.chars().all(|c| char::is_whitespace(c) || c == '!')
-        }).unwrap_or(false);
+        let preceeds_whitespace_and_paren = paren_idx
+            .map(|ip| {
+                let maybe_space = &contents[*i + "cfg".len()..ip];
+                maybe_space.chars().all(|c| char::is_whitespace(c) || c == '!')
+            })
+            .unwrap_or(false);
 
         succeeds_non_ident && preceeds_whitespace_and_paren
     });
@@ -213,12 +233,13 @@ fn parse_cfgs<'a>(contents: &'a str) -> Vec<(usize, &'a str)> {
                         return Some((i, &contents_from[..=j]));
                     }
                 }
-                _ => { }
+                _ => {}
             }
         }
 
         // if the parentheses are unbalanced just ignore this cfg -- it'll be caught when attempting
         // to run the compiler, and there's no real reason to lint it separately here
         None
-    }).collect()
+    })
+    .collect()
 }
